#include "stdio.h"
#include "string.h"
#include <stdlib.h>
#include <stdint.h>

#include "HWCtrlCorePadIO.h"

#include <ti/csl/soc.h>
#include <ti/csl/cslr_icss.h>
#include <ti/csl/csl_gpio.h>
#include <ti/csl/src/ip/mdio/V2/cslr_mdio.h>
#include <ti/csl/src/ip/mdio/V2/csl_mdio.h>
#include <ti/csl/src/ip/mdio/V2/csl_mdioAux.h>
#include <ti/drv/gpio/soc/GPIO_v1.h>

#include <ti/drv/pruss/pruicss.h>
#include <ti/drv/pruss/soc/pruicss_v1.h>

// TLK10xL Registers Definition
#define TLKPHY_BMCR_REG                   (0x0000)
#define TLKPHY_PHYCR_REG                  (0x0019)
#define TLKPHY_PHYRCR_REG                 (0x001F)
#define TLKPHY_CR1_REG                    (0x0009)
#define TLKPHY_CR3_REG                    (0x000B)

// TLK10xL PHYCR Register Bit Definition
#define TLKPHY_AUTOMDIX_ENABLE            (1u<<15)
#define TLKPHY_FORCEMDIX_ENABLE           (1u<<14)

// TLK10xL BMCR Register Bit Definition
#define TLKPHY_SPEED_SELECTION_100        (1u<<13) // 1 = 100Mps, 0 = 10 Mbps
#define TLKPHY_AUTONEGOTIATION_ENABLE     (1u<<12)
#define TLKPHY_RESTART_AUTONEG            (1u<<9)
#define TLKPHY_DUPLEX_MODE_FULL           (1u<<8)

// TLK10xL PHYRCR Register Bit Definition
#define TLKPHY_SOFT_RESTART               (1u<<14)

// TLK10xL CR1 Register Bit Definition
#define TLKPHY_LINK_LOSS_RECOVERY         (1u<<7)

// Pin Muxing Definition
#define PULLUDENABLE_ENABLE         (0U)
#define PULLUDENABLE_DISABLE        (1U)
#define PULLTYPESELECT_PULL_UP      (1U)
#define PULLTYPESELECT_PULL_DOWN    (0U)
#define INPUTENABLE_ENABLE          (1U)
#define INPUTENABLE_DISABLE         (0U)
#define SLEWCONTROL_FAST_SLEW       (0U)
#define SLEWCONTROL_SLOW_SLEW       (1U)
#define WAKEUPENABLE_ENABLE         (1U)
#define WAKEUPENABLE_DISABLE        (0U)

typedef struct
{
	uint32_t PinNum;
    uint8_t  MuxMode;
    uint8_t  PullUpEnable;
    uint8_t  PullTypeSelect;
    uint8_t  InputEnable;
    uint8_t  SlewControl;
    uint8_t  WakeupEnable;
} pad_config_t;

typedef struct
{
    uint32_t Pin;
    uint32_t BaseAddr;

} gpio_info_t;

PRUICSS_V1_Object prussObjects[PRUICCSS_INSTANCE_MAX-1];

PRUICSS_HwAttrs prussInitCfg[2] =
{
    {
        0x4b200000,
        0,
        CSL_MPU_PRUSS1_U_PRU0_CTRL_REGS,
        CSL_MPU_PRUSS1_U_PRU1_CTRL_REGS,
        CSL_MPU_PRUSS1_U_INTC_REGS,
        CSL_MPU_PRUSS1_U_CFG_REGS,
        CSL_MPU_PRUSS1_U_UART_REGS,
        CSL_MPU_PRUSS1_U_IEP_REGS,
        CSL_MPU_PRUSS1_U_ECAP_REGS,
        CSL_MPU_PRUSS1_U_MII_RT_CFG_REGS,
        CSL_MPU_PRUSS1_U_MII_MDIO_REGS,
        CSL_MPU_PRUSS1_U_DATA_RAM0_8KB_REGS,
        CSL_MPU_PRUSS1_U_DATA_RAM1_8KB_REGS,
        CSL_MPU_PRUSS1_U_INST_RAM0_12KB_REGS,
        CSL_MPU_PRUSS1_U_INST_RAM1_12KB_REGS,
        CSL_MPU_PRUSS1_U_DATA_RAM_32KB_REGS
    },
    {
        0x4b280000,
        0,
        CSL_MPU_PRUSS2_U_PRU0_CTRL_REGS,
        CSL_MPU_PRUSS2_U_PRU1_CTRL_REGS,
        CSL_MPU_PRUSS2_U_INTC_REGS,
        CSL_MPU_PRUSS2_U_CFG_REGS,
        CSL_MPU_PRUSS2_U_UART_REGS,
        CSL_MPU_PRUSS2_U_IEP_REGS,
        CSL_MPU_PRUSS2_U_ECAP_REGS,
        CSL_MPU_PRUSS2_U_MII_RT_CFG_REGS,
        CSL_MPU_PRUSS2_U_MII_MDIO_REGS,
        CSL_MPU_PRUSS2_U_DATA_RAM0_8KB_REGS,
        CSL_MPU_PRUSS2_U_DATA_RAM1_8KB_REGS,
        CSL_MPU_PRUSS2_U_INST_RAM0_12KB_REGS,
        CSL_MPU_PRUSS2_U_INST_RAM1_12KB_REGS,
        CSL_MPU_PRUSS2_U_DATA_RAM_32KB_REGS
    }
};


PRUICSS_Config pruss_config[PRUICCSS_INSTANCE_MAX] =
{
    {
        &prussObjects[0],
        &prussInitCfg[0],
    },
    {

         &prussObjects[1],
         &prussInitCfg[1],
    },
    {
         NULL,
         NULL,
    }
};


gpio_info_t PhyResetInfo[4] =
{
    {
        //GPIO_PRU1_ETH0_RESETn
        .Pin=6,
        .BaseAddr=CSL_MPU_GPIO5_REGS,
    },
    {
        //GPIO_PRU1_ETH1_RESETn
        .Pin=7,
        .BaseAddr=CSL_MPU_GPIO5_REGS,
    },
    {
        //GPIO_PRU2_ETH0_RESETn
        .Pin=8,
        .BaseAddr=CSL_MPU_GPIO5_REGS,
    },
    {
        //GPIO_PRU2_ETH1_RESETn
        .Pin=9,
        .BaseAddr=CSL_MPU_GPIO5_REGS,
    },
};


gpio_info_t PhyIntInfo[4] =
{
    {
        //PRU1_ETH0_INTn
        .Pin=28,
        .BaseAddr=CSL_MPU_GPIO3_REGS,
    },
    {
        //PRU1_ETH1_INTn
        .Pin=29,
        .BaseAddr=CSL_MPU_GPIO3_REGS,
    },
    {
        //PRU2_ETH0_INTn
        .Pin=30,
        .BaseAddr=CSL_MPU_GPIO3_REGS,
    },
    {
        //PRU2_ETH1_INTn
        .Pin=31,
        .BaseAddr=CSL_MPU_GPIO3_REGS,
    }
};


void PRUICSS_Delay(uint32_t delay)
{
    volatile uint32_t delay1 = delay*2;
    while (delay1--) ;
}


void PRUICSS_CTRLCorePadConfiguration(uint32_t baseAdd, pad_config_t *pad_config)
{
    uint32_t reg_val;

    reg_val = HWREG(baseAdd + (pad_config->PinNum));

    reg_val &= ~CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_MUXMODE_MASK;

    reg_val |= (uint32_t) pad_config->MuxMode <<
               CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_MUXMODE_SHIFT;

    if (pad_config->PullUpEnable != 0xFF)
    {
        reg_val &= ~CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_PULLUDENABLE_MASK;
        reg_val |= (uint32_t) pad_config->PullUpEnable <<
                   CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_PULLUDENABLE_SHIFT;
    }
    if (pad_config->PullTypeSelect != 0xFF)
    {
        reg_val &= ~CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_PULLTYPESELECT_MASK;
        reg_val |= (uint32_t) pad_config->PullTypeSelect <<
                   CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_PULLTYPESELECT_SHIFT;
    }
    if (pad_config->InputEnable != 0xFF)
    {
        reg_val &= ~CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_INPUTENABLE_MASK;
        reg_val |= (uint32_t) pad_config->InputEnable <<
                   CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_INPUTENABLE_SHIFT;
    }
    if (pad_config->SlewControl != 0xFF)
    {
        reg_val &= ~CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_SLEWCONTROL_MASK;
        reg_val |= (uint32_t) pad_config->SlewControl <<
                   CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_SLEWCONTROL_SHIFT;
    }
    if (pad_config->WakeupEnable != 0xFF)
    {
        reg_val &= ~CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_WAKEUPENABLE_MASK;
        reg_val |= (uint32_t) pad_config->WakeupEnable <<
                   CTRL_CORE_PAD_GPMC_AD0_GPMC_AD0_WAKEUPENABLE_SHIFT;
    }

    HWREG(baseAdd + (pad_config->PinNum)) = reg_val;
}


void PRUICSS_ConfigurePad(pad_config_t *pad_array, uint32_t elements)
{
    uint32_t i;

    for (i = 0; i < elements; ++i)
    {
        PRUICSS_CTRLCorePadConfiguration(CSL_MPU_CORE_PAD_IO_REGISTERS_REGS,
                                     &pad_array[i]);
    }
}


void PRUICSS_IEP_PinMux(uint8_t pruicssInstance)
{
	if (pruicssInstance == 1)
    {
        /*
        Port 0 - PRUSS1 IEP0

        RGMII0_RXC - PR1_MII_MT0_CLK
        RGMII0_RXD2 - PR1_MII0_TXEN
        RGMII0_RXD0 - PR1_MII0_TXD0
        RGMII0_RXD1 - PR1_MII0_TXD1
        RGMII0_RXD3 - PR1_MII0_TXD2
        RGMII0_RXCTL - PR1_MII0_TXD3

        UART3_TXD - PR1_MII_MR0_CLK
        UART3_RXD - PR1_MII0_RXDV
        RGMII0_TXD2 - PR1_MII0_RXER

        RGMII0_TXD0 - PR1_MII0_RXD0
        RGMII0_TXD1 - PR1_MII0_RXD1
        RGMII0_TXCTL - PR1_MII0_RXD2
        RGMII0_TXC - PR1_MII0_RXD3

        RGMII0_TXD3 - PR1_MII0_CRS
        MDIO_CLK - PR1_MII0_COL
        MDIO_DATA - PR1_MII0_RXLINK

        VIN2A_D10 - PR1_MDIO_MDCLK
        VIN2A_D11 - PR1_MDIO_DATA
        */
        pad_config_t pruss1_iep0_pad_config[] = {

        /* MDIO (2 pads) */
        {CTRL_CORE_PAD_VIN2A_D10,
         CTRL_CORE_PAD_VIN2A_D10_VIN2A_D10_MUXMODE_PR1_MDIO_MDCLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_UP, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D11,
         CTRL_CORE_PAD_VIN2A_D11_VIN2A_D11_MUXMODE_PR1_MDIO_DATA_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_UP, INPUTENABLE_ENABLE, 0xff, WAKEUPENABLE_DISABLE},

        /*Pin_num,Muxmode,pullud_enable,pull_typeselect,io,slewrate,wakeupmode*/
        {CTRL_CORE_PAD_RGMII0_RXC,
         CTRL_CORE_PAD_RGMII0_RXC_RGMII0_RXC_MUXMODE_PR1_MII_MT0_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_RXD2,
         CTRL_CORE_PAD_RGMII0_RXD2_RGMII0_RXD2_MUXMODE_PR1_MII0_TXEN_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_RXD0,
         CTRL_CORE_PAD_RGMII0_RXD0_RGMII0_RXD0_MUXMODE_PR1_MII0_TXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_RXD1,
         CTRL_CORE_PAD_RGMII0_RXD1_RGMII0_RXD1_MUXMODE_PR1_MII0_TXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_RXD3,
         CTRL_CORE_PAD_RGMII0_RXD3_RGMII0_RXD3_MUXMODE_PR1_MII0_TXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_RXCTL,
         CTRL_CORE_PAD_RGMII0_RXCTL_RGMII0_RXCTL_MUXMODE_PR1_MII0_TXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_UART3_TXD,
         CTRL_CORE_PAD_UART3_TXD_UART3_TXD_MUXMODE_PR1_MII_MR0_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_UART3_RXD,
         CTRL_CORE_PAD_UART3_RXD_UART3_RXD_MUXMODE_PR1_MII0_RXDV_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_TXD2,
         CTRL_CORE_PAD_RGMII0_TXD2_RGMII0_TXD2_MUXMODE_PR1_MII0_RXER_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_TXD0,
         CTRL_CORE_PAD_RGMII0_TXD0_RGMII0_TXD0_MUXMODE_PR1_MII0_RXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_TXD1,
         CTRL_CORE_PAD_RGMII0_TXD1_RGMII0_TXD1_MUXMODE_PR1_MII0_RXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_TXCTL,
         CTRL_CORE_PAD_RGMII0_TXCTL_RGMII0_TXCTL_MUXMODE_PR1_MII0_RXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_TXC,
         CTRL_CORE_PAD_RGMII0_TXC_RGMII0_TXC_MUXMODE_PR1_MII0_RXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_RGMII0_TXD3,
         CTRL_CORE_PAD_RGMII0_TXD3_RGMII0_TXD3_MUXMODE_PR1_MII0_CRS_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},
#if 0
       {CTRL_CORE_PAD_MDIO_MCLK,
       CTRL_CORE_PAD_MDIO_MCLK_MDIO_MCLK_MUXMODE_PR1_MII0_COL_11,
       PULLUDENABLE_ENABLE,
       PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
       WAKEUPENABLE_DISABLE},
#else
        {CTRL_CORE_PAD_MDIO_MCLK,
         0xff,
         0xff,
         0xff, 0xff, 0xff,
         0xff},
#endif
        {CTRL_CORE_PAD_MDIO_D,
         CTRL_CORE_PAD_MDIO_D_MDIO_D_MUXMODE_PR1_MII0_RXLINK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE}
    };

    /*configure the rgmii pad*/
    PRUICSS_ConfigurePad(pruss1_iep0_pad_config, sizeof (pruss1_iep0_pad_config) /
            sizeof (pruss1_iep0_pad_config[0]));


    /*
      Port 1 - PRUSS1 IEP1

      VIN2A_D6 - PR1_MII_MT1_CLK
      VIN2A_D7 - PR1_MII1_TXEN
      VIN2A_D13 - PR1_MII1_TXD0
      VIN2A_D12 - PR1_MII1_TXD1
      VIN2A_D9 - PR1_MII1_TXD2
      VIN2A_D8 - PR1_MII1_TXD3

      VIN2A_D14 - PR1_MII_MR1_CLK
      VIN2A_D15 - PR1_MII1_RXDV
      VIN2A_D20 - PR1_MII1_RXER

      VIN2A_D19 - PR1_MII1_RXD0
      VIN2A_D18 - PR1_MII1_RXD1
      VIN2A_D17 - PR1_MII1_RXD2
      VIN2A_D16 - PR1_MII1_RXD3

      VIN2A_D23 - PR1_MII1_CRS
      VIN2A_D22 - PR1_MII1_COL
      VIN2A_D21 - PR1_MII1_RXLINK

    */

        pad_config_t pruss1_iep1_pad_config[] = {

      /*Pin_num,Muxmode,pullud_enable,pull_typeselect,io,slewrate,wakeupmode*/
        {CTRL_CORE_PAD_VIN2A_D6,
         CTRL_CORE_PAD_VIN2A_D6_VIN2A_D6_MUXMODE_PR1_MII_MT1_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D7,
         CTRL_CORE_PAD_VIN2A_D7_VIN2A_D7_MUXMODE_PR1_MII1_TXEN_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D13,
         CTRL_CORE_PAD_VIN2A_D13_VIN2A_D13_MUXMODE_PR1_MII1_TXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D12,
         CTRL_CORE_PAD_VIN2A_D12_VIN2A_D12_MUXMODE_PR1_MII1_TXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D9,
         CTRL_CORE_PAD_VIN2A_D9_VIN2A_D9_MUXMODE_PR1_MII1_TXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D8,
         CTRL_CORE_PAD_VIN2A_D8_VIN2A_D8_MUXMODE_PR1_MII1_TXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D14,
         CTRL_CORE_PAD_VIN2A_D14_VIN2A_D14_MUXMODE_PR1_MII_MR1_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D15,
         CTRL_CORE_PAD_VIN2A_D15_VIN2A_D15_MUXMODE_PR1_MII1_RXDV_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D20,
         CTRL_CORE_PAD_VIN2A_D20_VIN2A_D20_MUXMODE_PR1_MII1_RXER_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D19,
         CTRL_CORE_PAD_VIN2A_D19_VIN2A_D19_MUXMODE_PR1_MII1_RXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D18,
         CTRL_CORE_PAD_VIN2A_D18_VIN2A_D18_MUXMODE_PR1_MII1_RXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D17,
         CTRL_CORE_PAD_VIN2A_D17_VIN2A_D17_MUXMODE_PR1_MII1_RXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D16,
         CTRL_CORE_PAD_VIN2A_D16_VIN2A_D16_MUXMODE_PR1_MII1_RXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_VIN2A_D23,
         CTRL_CORE_PAD_VIN2A_D23_VIN2A_D23_MUXMODE_PR1_MII1_CRS_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},
#if 0
        {CTRL_CORE_PAD_VIN2A_D22,
        CTRL_CORE_PAD_VIN2A_D22_VIN2A_D22_MUXMODE_PR1_MII1_COL_11,
        PULLUDENABLE_ENABLE,
        PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
        WAKEUPENABLE_DISABLE},
#else
        {CTRL_CORE_PAD_VIN2A_D22,
        0xff,
        0xff,
        0xff, 0xff, 0xff,
        0xff},
#endif
        {CTRL_CORE_PAD_VIN2A_D21,
         CTRL_CORE_PAD_VIN2A_D21_VIN2A_D21_MUXMODE_PR1_MII1_RXLINK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},
    };

    /*configure the rgmii pad*/
    PRUICSS_ConfigurePad(pruss1_iep1_pad_config, sizeof (pruss1_iep1_pad_config) /
                  sizeof (pruss1_iep1_pad_config[0]));



  }
  else
  {
    /*
    Port2 - PRUSS2 IEP 0

    MCASP1_AXR1 - PR2_MII_MT0_CLK
    MCASP1_AXR8 - PR2_MII0_TXEN
    MCASP1_AXR12 - PR2_MII0_TXD0
    MCASP1_AXR11 - PR2_MII0_TXD1
    MCASP1_AXR10 - PR2_MII0_TXD2
    MCASP1_AXR9 - PR2_MII0_TXD3

    MCASP1_AXR13 - PR2_MII_MR0_CLK
    MCASP1_AXR14 - PR2_MII0_RXDV
    MCASP1_AXR0 -PR2_MII0_RXER

    MCASP2_AXR2 -PR2_MII0_RXD0
    MCASP2_FSX - PR2_MII0_RXD1
    MCASP2_ACLKX - PR2_MII0_RXD2
    MCASP1_AXR15 - PR2_MII0_RXD3

    MCASP3_ACLKX - PR2_MII0_CRS
    MCASP3_FSX - PR2_MII0_COL
    MCASP2_AXR3 - PR2_MII0_RXLINK

    MCASP1_ACLKX - PR2_MDIO_MDCLK
    MCASP1_FSX - PR2_MDIO_DATA

    */

    pad_config_t pruss2_iep0_pad_config[] = {

        /* MDIO (2 pads) */
        {CTRL_CORE_PAD_MCASP1_ACLKX,
         CTRL_CORE_PAD_MCASP1_ACLKX_MCASP1_ACLKX_MUXMODE_PR2_MDIO_MDCLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_UP, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_FSX,
         CTRL_CORE_PAD_MCASP1_FSX_MCASP1_FSX_MUXMODE_PR2_MDIO_DATA_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_UP, INPUTENABLE_ENABLE, 0xff, WAKEUPENABLE_DISABLE},

        /*Pin_num,Muxmode,pullud_enable,pull_typeselect,io,slewrate,wakeupmode*/
        {CTRL_CORE_PAD_MCASP1_AXR1,
         CTRL_CORE_PAD_MCASP1_AXR1_MCASP1_AXR1_MUXMODE_PR2_MII_MT0_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR8,
         CTRL_CORE_PAD_MCASP1_AXR8_MCASP1_AXR8_MUXMODE_PR2_MII0_TXEN_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR12,
         CTRL_CORE_PAD_MCASP1_AXR12_MCASP1_AXR12_MUXMODE_PR2_MII0_TXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR11,
         CTRL_CORE_PAD_MCASP1_AXR11_MCASP1_AXR11_MUXMODE_PR2_MII0_TXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR10,
         CTRL_CORE_PAD_MCASP1_AXR10_MCASP1_AXR10_MUXMODE_PR2_MII0_TXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR9,
         CTRL_CORE_PAD_MCASP1_AXR9_MCASP1_AXR9_MUXMODE_PR2_MII0_TXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR13,
         CTRL_CORE_PAD_MCASP1_AXR13_MCASP1_AXR13_MUXMODE_PR2_MII_MR0_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR14,
         CTRL_CORE_PAD_MCASP1_AXR14_MCASP1_AXR14_MUXMODE_PR2_MII0_RXDV_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR0,
         CTRL_CORE_PAD_MCASP1_AXR0_MCASP1_AXR0_MUXMODE_PR2_MII0_RXER_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP2_AXR2,
         CTRL_CORE_PAD_MCASP2_AXR2_MCASP2_AXR2_MUXMODE_PR2_MII0_RXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP2_FSX,
         CTRL_CORE_PAD_MCASP2_FSX_MCASP2_FSX_MUXMODE_PR2_MII0_RXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP2_ACLKX,
         CTRL_CORE_PAD_MCASP2_ACLKX_MCASP2_ACLKX_MUXMODE_PR2_MII0_RXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP1_AXR15,
         CTRL_CORE_PAD_MCASP1_AXR15_MCASP1_AXR15_MUXMODE_PR2_MII0_RXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MCASP3_ACLKX,
         CTRL_CORE_PAD_MCASP3_ACLKX_MCASP3_ACLKX_MUXMODE_PR2_MII0_CRS_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},
#if 0
        {CTRL_CORE_PAD_MCASP3_FSX,
         CTRL_CORE_PAD_MCASP3_FSX_MCASP3_FSX_MUXMODE_PR2_MII0_COL_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},
#else
       {CTRL_CORE_PAD_MCASP3_FSX,
        0xff,
        0xff,
        0xff, 0xff, 0xff,
        0xff},
#endif

#if 0
        {CTRL_CORE_PAD_MCASP1_AXR3,
         CTRL_CORE_PAD_MCASP2_AXR3_MCASP2_AXR3_MUXMODE_PR2_MII0_RXLINK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE}
#else
        {CTRL_CORE_PAD_MCASP1_AXR3,
         0xff,
         0xff,
         0xff, 0xff, 0xff,
         0xff},
#endif
    };

    /*configure the pruss2_iep0 pad*/
    PRUICSS_ConfigurePad(pruss2_iep0_pad_config, sizeof (pruss2_iep0_pad_config) /
                sizeof (pruss2_iep0_pad_config[0]));

    /*
    Port3 -   PRUSS2 IEP 1

    GPIO6_10 - PR2_MII_MT1_CLK
    GPIO6_11 - PR2_MII1_TXEN
    MMC3_DAT1 - PR2_MII1_TXD0
    MMC3_DAT0 - PR2_MII1_TXD1
    MMC3_CMD - PR2_MII1_TXD2
    MMC3_CLK - PR2_MII1_TXD3

    MMC3_DAT2 - PR2_MII_MR1_CLK
    MMC3_DAT3 - PR2_MII1_RXDV
    MCASP3_AXR0 -PR2_MII1_RXER

    MMC3_DAT7 -PR2_MII1_RXD0
    MMC3_DAT6 - PR2_MII1_RXD1
    MMC3_DAT5 - PR2_MII1_RXD2
    MMC3_DAT4 - PR2_MII1_RXD3

    XREF_CLK1 - PR2_MII1_CRS
    XREF_CLK0 - PR2_MII1_COL
    MCASP3_AXR1 - PR2_MII1_RXLINK

    */
    pad_config_t pruss2_iep1_pad_config[] = {

        /*Pin_num,Muxmode,pullud_enable,pull_typeselect,io,slewrate,wakeupmode*/
        {CTRL_CORE_PAD_GPIO6_10,
         CTRL_CORE_PAD_GPIO6_10_GPIO6_10_MUXMODE_PR2_MII_MT1_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_GPIO6_11,
         CTRL_CORE_PAD_GPIO6_11_GPIO6_11_MUXMODE_PR2_MII1_TXEN_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT1,
         CTRL_CORE_PAD_MMC3_DAT1_MMC3_DAT1_MUXMODE_PR2_MII1_TXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT0,
         CTRL_CORE_PAD_MMC3_DAT0_MMC3_DAT0_MUXMODE_PR2_MII1_TXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_CMD,
         CTRL_CORE_PAD_MMC3_CMD_MMC3_CMD_MUXMODE_PR2_MII1_TXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_CLK,
         CTRL_CORE_PAD_MMC3_CLK_MMC3_CLK_MUXMODE_PR2_MII1_TXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_DISABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT2,
         CTRL_CORE_PAD_MMC3_DAT2_MMC3_DAT2_MUXMODE_PR2_MII_MR1_CLK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT3,
         CTRL_CORE_PAD_MMC3_DAT3_MMC3_DAT3_MUXMODE_PR2_MII1_RXDV_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

         {CTRL_CORE_PAD_MCASP3_AXR0,
         CTRL_CORE_PAD_MCASP3_AXR0_MCASP3_AXR0_MUXMODE_PR2_MII1_RXER_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT7,
         CTRL_CORE_PAD_MMC3_DAT7_MMC3_DAT7_MUXMODE_PR2_MII1_RXD0_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT6,
         CTRL_CORE_PAD_MMC3_DAT6_MMC3_DAT6_MUXMODE_PR2_MII1_RXD1_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT5,
         CTRL_CORE_PAD_MMC3_DAT5_MMC3_DAT5_MUXMODE_PR2_MII1_RXD2_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_MMC3_DAT4,
         CTRL_CORE_PAD_MMC3_DAT4_MMC3_DAT4_MUXMODE_PR2_MII1_RXD3_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},

        {CTRL_CORE_PAD_XREF_CLK1,
         CTRL_CORE_PAD_XREF_CLK1_XREF_CLK1_MUXMODE_PR2_MII1_CRS_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},
#if 0
        {CTRL_CORE_PAD_XREF_CLK0,
         CTRL_CORE_PAD_XREF_CLK0_XREF_CLK0_MUXMODE_PR2_MII1_COL_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE},
#else
       {CTRL_CORE_PAD_XREF_CLK0,
        0xff,
        0xff,
        0xff, 0xff, 0xff,
        0xff},
#endif

#if 0
        {CTRL_CORE_PAD_MCASP3_AXR1,
         CTRL_CORE_PAD_MCASP3_AXR1_MCASP3_AXR1_MUXMODE_PR2_MII1_RXLINK_11,
         PULLUDENABLE_ENABLE,
         PULLTYPESELECT_PULL_DOWN, INPUTENABLE_ENABLE, 0xff,
         WAKEUPENABLE_DISABLE}
#else
        {CTRL_CORE_PAD_MCASP3_AXR1,
         0xff,
         0xff,
         0xff, 0xff, 0xff,
         0xff},
#endif

    };

    /*configure the pruss2_iep0 pad*/
    PRUICSS_ConfigurePad(pruss2_iep1_pad_config, sizeof (pruss2_iep1_pad_config) /
                sizeof (pruss2_iep1_pad_config[0]));

    }

}


void PRUICSS_IEP_Initialization(uint8_t pruicssInstance)
{
    uint32_t pruss_base = 0;

    // Perform all PRU ICSS pin muxing
    PRUICSS_IEP_PinMux(pruicssInstance);

    if (pruicssInstance == 1)
    {
        pruss_base = CSL_MPU_PRUSS1_U_CFG_REGS;

        //Enable the PRUSS clock
        HWREG(CSL_MPU_L4PER_CM_CORE_REGS + CM_L4PER2_PRUSS1_CLKCTRL) = 0;
    }
    else
    {
        pruss_base = CSL_MPU_PRUSS2_U_CFG_REGS;

        //Enable the PRUSS clock
        HWREG(CSL_MPU_L4PER_CM_CORE_REGS + CM_L4PER2_PRUSS2_CLKCTRL) = 0;
    }

    PRUICSS_Delay(500);

    if (pruicssInstance==1)
    {
    	HWREG(CSL_MPU_L4PER_CM_CORE_REGS + CM_L4PER2_PRUSS1_CLKCTRL) = 2;
    	while(0 != (HWREG(CSL_MPU_L4PER_CM_CORE_REGS + CM_L4PER2_PRUSS1_CLKCTRL) & 0x00030000));
    }
    else
    {
    	HWREG(CSL_MPU_L4PER_CM_CORE_REGS + CM_L4PER2_PRUSS2_CLKCTRL) = 2;
    	while(0 != (HWREG(CSL_MPU_L4PER_CM_CORE_REGS + CM_L4PER2_PRUSS2_CLKCTRL) & 0x00030000));
    }

    while(0x6100 != ((HWREG(CSL_MPU_L4PER_CM_CORE_REGS + CM_L4PER2_CLKSTCTRL))&0x6100)); //ICSS_CLK, ICSS_IEP_CLK

    HWREG(pruss_base + CSL_ICSSM_CFG_SYSCFG) = 0x06; // NO_STANDBY
    PRUICSS_Delay(500);
}


void PRUICSS_IEP_MDIOInitialization(uint8_t pruicssInstance)
{
    uint32_t pruss_base = 0;

    if (pruicssInstance == 1)
    {
        pruss_base = CSL_MPU_PRUSS1_U_MII_MDIO_REGS;
    }
    else
    {
        pruss_base = CSL_MPU_PRUSS2_U_MII_MDIO_REGS;
    }

    // Initialize the MDIO pin
    HWREG(pruss_base + CSL_ICSSM_MII_MDIO_CONTROL) = 0x40000007; // 200/(0x7+1) = 25 MHz
    PRUICSS_Delay(5000);
}


void PRUICSS_PhyResetPulse()
{
    int i = 0;
    int numPorts = 4;

    for(i = 0; i < numPorts; i++)
    {
        GPIOModuleEnable(PhyResetInfo[i].BaseAddr);
        GPIOPinWrite(PhyResetInfo[i].BaseAddr, PhyResetInfo[i].Pin, GPIO_PIN_LOW);
        PRUICSS_Delay(1000);
        GPIOPinWrite(PhyResetInfo[i].BaseAddr, PhyResetInfo[i].Pin, GPIO_PIN_HIGH);
        PRUICSS_Delay(1000);
    }
}


void PRUICSS_PhyResetConfig()
{
    int i = 0;
    int numPorts = 4;

    for(i = 0; i < numPorts; i++)
    {
        GPIOModuleEnable(PhyResetInfo[i].BaseAddr);
        GPIODirModeSet(PhyResetInfo[i].BaseAddr, PhyResetInfo[i].Pin, GPIO_DIR_OUTPUT);

        GPIOPinWrite(PhyResetInfo[i].BaseAddr, PhyResetInfo[i].Pin, GPIO_PIN_HIGH);
        PRUICSS_Delay(1000);
    }
}


void PRUICSS_PhyIntConfig()
{
    int i = 0;
    int numPorts = 4;

    for(i = 0; i < numPorts; i++)
    {
        GPIOModuleEnable(PhyIntInfo[i].BaseAddr);
        GPIODirModeSet(PhyIntInfo[i].BaseAddr, PhyIntInfo[i].Pin, GPIO_DIR_INPUT);
    }
}


int PRUICSS_InitBoth()
{
	// Configure PHYs reset pins
	PRUICSS_PhyResetConfig();

    // Pulse the PHYs reset pin
    PRUICSS_PhyResetPulse();
    PRUICSS_Delay(2000000);

    // PHYs interrupt lines are configured as GPIO Input in order to avoid any
    // spurious power down signal to TLK105L
    PRUICSS_PhyIntConfig();

    // Initialize the IEP
    PRUICSS_IEP_Initialization(1);
    PRUICSS_IEP_Initialization(2);

    // Initialize the MDIO
    PRUICSS_IEP_MDIOInitialization(1);
    PRUICSS_IEP_MDIOInitialization(2);

    return (0);
}

